# -*- coding: utf8 -*-
from __future__ import unicode_literals

import logging

from openpyxl import load_workbook

from driverng import MessageV2, Worklist

logger = logging.getLogger("driverng.drivers.stp7000")


class Driver(MessageV2):
    SPEC = {"code": "stp7000", "label": "Extracteur STP7000"}
    BINARY_PARSING = True

    def parse(self, data):
        """
        Convert the .xlsx file (data) into a worksheet using openpyxl.
        """
        workbook = load_workbook(data, read_only=True, data_only=True)
        return workbook.active

    def get_geometry_format(self, value):
        """
        Retrieve geometry format depending the value of the axis reference.

        Returns:
            str: "n0" or "n1" if format is interger, "al" if format is alphabetical
        """
        try:
            int(value)
            return f"n{value}"
        except ValueError:
            return "al"

    def get_geometry_height(self, message):
        """
        Retrieve geometry height depending the value of the row axis reference.

        Returns:
            int: number of row
        """
        index_row = 0
        for value in message.iter_rows(
            min_row=3, min_col=1, max_col=1, values_only=True
        ):
            if value[0] is None:
                return index_row
            index_row += 1
        return index_row

    def get_geometry_width(self, message):
        """
        Retrieve geometry width depending the value of the column axis reference.

        Returns:
            int: number of column
        """
        index_col = 0
        for values in message.iter_rows(
            min_col=2, min_row=2, max_row=2, values_only=True
        ):
            for value in values:
                if value is None:
                    return index_col
                index_col += 1
        return index_col

    def decode_message(self, message):
        """
        message = worksheet given by "parse" function.
        La worklist obtenue se présente de la façon suivante :

        worklist = {
            "context": "result",
                "origin": "plandeplaque",
                "samples": [
                    {
                        "geometry": {
                            "format_abscissa": "n1",
                            "format_ordinate": "al",
                            "height": 12,
                            "width": 8,
                        },
                        "sample_id": "12345",
                        "type": "patient",
                        "wells": [
                            {
                                "abscissa": "1",
                                "barcode": "19E340041469",
                                "ordinate": "A",
                                "well_type": 0,
                            },
                            {
                                "abscissa": "2",
                                "barcode": "19E320083243",
                                "ordinate": "A",
                                "well_type": 0,
                            },
                            {
                                "abscissa": "3",
                                "barcode": "19E111106848",
                                "ordinate": "A",
                                "well_type": 0,
                            },
                        ],
                    }
                ],
            }

        Type de position sur la plaque (well_type) :
        0 : Position (puit) patient
        1 : Position (puit) de contrôle
        """

        worklist = Worklist(context="result", origin="plandeplaque")

        # Check if the Excel file contains sample_id value.
        if message.cell(row=1, column=2).value is None:
            logger.error(
                "Le fichier Excel n'a pas pu être traité, car aucun sample id n'a été trouvé"
            )
            return worklist

        sample = worklist.get_or_create(
            type="patient", sample_id=str(message.cell(row=1, column=2).value)
        )

        # Check if the Excel file contains "tube" value in first position.
        if message.cell(row=3, column=2).value is None:
            logger.error("La plaque ne contient aucun échantillon")
            return worklist

        # Set Geometry.
        sample.geometry.height = self.get_geometry_height(message)
        sample.geometry.width = self.get_geometry_width(message)
        sample.geometry.format_abscissa = self.get_geometry_format(
            message.cell(row=2, column=2).value
        )
        sample.geometry.format_ordinate = self.get_geometry_format(
            message.cell(row=3, column=1).value
        )

        # qc_count alows to stop creating wells when all wells of the board are
        # not fulfill (cf: example4.xlsx file in tests directory).
        # A board has two quality control wells after all fulfill tube wells.
        qc_count = 0
        for index_row, values in enumerate(
            list(message.iter_rows(min_row=3, min_col=2, values_only=True))
        ):
            if qc_count >= 2:
                break
            for index_col, value in enumerate(values):
                if qc_count >= 2:
                    break
                sample.wells.get_or_create(
                    barcode=str(value) if value is not None else "",
                    abscissa=message.cell(2, index_col + 2).value,
                    ordinate=message.cell(index_row + 3, 1).value,
                    well_type=0 if value is not None else 1,
                )
                if value is None:
                    qc_count += 1

        return worklist
